@@ -1,33 +1,40 @@
 package Apache;
+
 use strict;
+use mod_perl 1.17_01;
 use Exporter ();
 use Apache::Constants qw(OK DECLINED);
+use Apache::Connection ();
+use Apache::Server ();
+use Apache::SIG ();
 
 @Apache::EXPORT_OK = qw(exit warn);
-$Apache::VERSION = "1.20";
 
 *import = \&Exporter::import;
 
-unless(defined &bootstrap) {
-    require DynaLoader;
-    @Apache::ISA = qw(DynaLoader);
-}
-
 if (caller eq "CGI::Apache") {
     #we must die here outside of httpd so CGI::Switch works
-    bootstrap Apache $Apache::VERSION;
+    die unless $ENV{MOD_PERL};
 }
-else {
-    eval { bootstrap Apache $Apache::VERSION; } 
-}
-if($@) {
-    die "$@\n" if exists $ENV{MOD_PERL};
-    warn "warning: can't `bootstrap Apache $Apache::VERSION' outside of httpd\n";
+ 
+Apache::SIG->set;
+
+{
+    no strict;
+    $VERSION = "1.26";
+    __PACKAGE__->mod_perl::boot($VERSION);
 }
 
-sub module {
-    my($self, $module) = @_;
-    eval "require $module;";
+if($ENV{MOD_PERL} && perl_hook("Sections")) {
+    *Apache::ReadConfig:: = \%ApacheReadConfig::;
+
+    if(Apache::Constants::MODULE_MAGIC_NUMBER() >= 19971026) {
+	*Apache::httpd_conf = sub {
+	    shift;
+	    push @Apache::ReadConfig::PerlConfig,
+	    map "$_\n", @_;
+	};
+    }
 }
 
 sub parse_args {
@@ -41,7 +48,7 @@ sub parse_args {
 
 sub content {
     my($r) = @_;
-    my $ct = $r->header_in("Content-type");
+    my $ct = $r->header_in("Content-type") || "";
     return unless $ct eq "application/x-www-form-urlencoded";
     my $buff;
     $r->read($buff, $r->header_in("Content-length"));
@@ -49,17 +56,13 @@ sub content {
 }
 
 sub args {
-    my($r) = @_;
-    parse_args(wantarray, $r->query_string);
+    my($r, $val) = @_;
+    parse_args(wantarray, 
+	       $val ? $r->query_string($val) : $r->query_string);
 }
 
-sub cgi_var {
-    my($r, $key) = @_;
-    my $val = $r->cgi_env($key);
-    return $val;
-}
+*READ = \&read unless defined &READ;
 
-*READ = \&read;
 sub read {
     my($r, $bufsiz) = @_[0,2];
     my($nrd, $buf, $total);
@@ -82,13 +85,60 @@ sub read {
 	}
 	else {
 	    $_[1] = undef;
+	    last;
 	}
     }
     $r->kill_timeout;
     return $total;
 }
 
-sub GETC { my $c; shift->read($c,1); $c; }
+sub new_read {
+    my($r, $bufsiz) = @_[0,2];
+    my($nrd, $buf, $total);
+    $nrd = $total = 0;
+    $buf = "";
+    $_[1] ||= "";
+
+    if(my $rv = $r->setup_client_block) {
+	$r->log_error("Apache->read: setup_client_block returned $rv");
+	die $rv;
+    }
+
+    #XXX: must set r->read_length to 0 here,
+    #since this read() method may be called in loop
+    #in which case, the second time in, should_client_block() 
+    #thinks we've already read the request body and returns 0
+    $r->read_length(0); 
+
+    unless($r->should_client_block) {
+	my $rl = $r->read_length;
+	$r->log_error("Apache->read: should_client_block returned 0 (rl=$rl)");
+	return 0;
+    }
+
+    $r->hard_timeout("Apache->read");
+    
+    while($bufsiz) {
+	$nrd = $r->get_client_block($buf, $bufsiz) || 0;
+	if(defined $nrd and $nrd > 0) {
+	    $bufsiz -= $nrd;
+	    $_[1] .= $buf;
+ 	    #substr($_[1], $total, $nrd) = $buf;
+	    $total += $nrd;
+	    $r->reset_timeout;
+	    next if $bufsiz;
+	    last;
+	}
+	else {
+	    $_[1] = undef;
+	    last;
+	}
+    }
+    $r->kill_timeout;
+    return $total;
+}
+
+sub GETC { my $c; shift->READ($c,1); $c; }
 
 #shouldn't use <STDIN> anyhow, but we'll be nice
 sub READLINE { 
@@ -98,13 +148,18 @@ sub READLINE {
     $line;
 }
 
-*PRINT = \&print;
-
 sub PRINTF {
     my $r = shift;
     my $fmt = shift;
     $r->print(sprintf($fmt, @_));
 }
+*printf = \&PRINTF;
+
+sub WRITE {
+    my($r, $buff, $length, $offset) = @_;
+    my $send = substr($buff, $offset, $length);
+    $r->print($send);
+}
 
 sub send_cgi_header {
     my($r, $headers) = @_;
@@ -134,29 +189,6 @@ sub send_cgi_header {
     }
 }
 
-sub as_string {
-    my($r) = @_;
-    my($k,$v,@retval);
-    my(%headers_in) = $r->headers_in;
-
-    push @retval, $r->the_request;
-    while(($k,$v) = each %headers_in) {
-	push @retval, "$k: $v";
-    }
-
-    push @retval, "";
-
-    push @retval, $r->status_line;
-    for (qw(err_headers_out headers_out)) {
-	my(%headers_out) = $r->$_();
-
-	while(($k,$v) = each %headers_out) {
-	    push @retval, "$k: $v";
-	}
-    }    
-    join "\n", @retval, "";
-}
-
 sub TIEHANDLE {
     my($class, $r) = @_;
     $r ||= Apache->request;
@@ -226,6 +258,11 @@ structure or C<undef> if there is no previous request.
 This method returns a blessed reference to the next (internal) request
 structure or C<undef> if there is no next request.
 
+=item $r->last
+
+This method returns a blessed reference to the last (internal) request
+structure.  Handy for logging modules.
+
 =item $r->is_main
 
 Returns true if the current request object is for the main request.
@@ -356,6 +393,10 @@ headers.  This can be used to initialize a perl hash, or one could use
 the $r->header_in() method (described below) to retrieve a specific
 header value directly.
 
+Will return a I<HASH> reference blessed into the I<Apache::Table>
+class when called in a scalar context with no "key" argument. This
+requires I<Apache::Table>.
+
 =item $r->header_in( $header_name, [$value] )
 
 Return the value of a client header.  Can be used like this:
@@ -373,15 +414,9 @@ returned.  When called in a list context, a list of parsed I<key> =>
 I<value> pairs are returned.  *NOTE*: you can only ask for this once,
 as the entire body is read from the client.
 
-=item $r->read_client_block($buf, $bytes_to_read)
-
-Read from the entity body sent by the client.  Example of use:
-
-   $r->read_client_block($buf, $r->header_in('Content-length'));
-
 =item $r->read($buf, $bytes_to_read)
 
-This method uses read_client_block() to read data from the client, 
+This method is used to read data from the client, 
 looping until it gets all of C<$bytes_to_read> or a timeout happens.
 
 In addition, this method sets a timeout before reading with
@@ -438,6 +473,10 @@ The dotted decimal representation of the remote client's IP address.
 This is set by then server when the connection record is created so
 is always defined.
 
+You can also set this value by providing an argument to it. This is
+helpful if your server is behind a squid accelerator proxy which add
+a HTTP_X_FORWARDED_FOR header.
+
 =item $c->local_addr
 
 A packed SOCKADDR_IN in the same format as returned by
@@ -517,6 +556,10 @@ C<PerlSetVar> directive.
 
    my $val = $r->dir_config('Key');
 
+Will return a I<HASH> reference blessed into the
+I<Apache::Table> class when called in a scalar context with no
+"key" argument. See I<Apache::Table>.
+
 =item $r->requires
 
 Returns an array reference of hash references, containing information
@@ -566,6 +609,10 @@ module provides the constants to check against.
 		      $filename);
    }
 
+=item $r->get_server_port
+
+Returns the port number on which the server is listening.
+
 =item $s = $r->server
 
 Return a reference to the server info object (blessed into the
@@ -595,7 +642,7 @@ Returns true if this is a virtual server.
 
 =item $s->names
 
-Returns the wild-carded names for HostAlias servers. 
+Returns the wild-carded names for ServerAlias servers. 
 
 =item $s->warn
 
@@ -659,6 +706,10 @@ writing handlers in mod_perl you can use Perl variables for this.
    $r->notes("MY_HANDLER", OK);
    $val = $r->notes("MY_HANDLER");
 
+Will return a I<HASH> reference blessed into the I<Apache::Table>
+class when called in a scalar context with no "key" argument. This
+requires I<Apache::Table>.
+
 =item $r->subprocess_env( $key, [$value] )
 
 Return the value of a named entry in the Apache C<subprocess_env>
@@ -679,6 +730,10 @@ you can then write in your .shtml document:
    Sorry
    <!--#endif -->
 
+Will return a I<HASH> reference blessed into the I<Apache::Table>
+class when called in a scalar context with no "key" argument. This
+requires I<Apache::Table>.
+
 =item $r->content_type( [$newval] )
 
 Get or set the content type being sent to the client.  Content types
@@ -695,10 +750,11 @@ Get or set the content encoding.  Content encodings are string like
 "gzip" or "compress".  This correspond to the "Content-Encoding"
 header in the HTTP protocol.
 
-=item $r->content_language( [$newval] )
+=item $r->content_languages( [$array_ref] )
 
-Get or set the content language.  The content language corresponds to the
-"Content-Language" HTTP header and is a string like "en" or "no".
+Get or set the content languages.  The content language corresponds to the
+"Content-Language" HTTP header and is an array reference containing strings
+such as "en" or "no".
 
 =item $r->status( $integer )
 
@@ -719,6 +775,10 @@ headers.  This can be used to initialize a perl hash, or one could use
 the $r->header_out() method (described below) to retrieve or set a specific
 header value directly.
 
+Will return a I<HASH> reference blessed into the I<Apache::Table>
+class when called in a scalar context with no "key" argument. This
+requires I<Apache::Table>.
+
 =item $r->header_out( $header, $value )
 
 Change the value of a response header, or create a new one.  You
@@ -739,6 +799,10 @@ The difference between headers_out and err_headers_out is that the
 latter are printed even on error, and persist across internal redirects
 (so the headers printed for ErrorDocument handlers will have them).
 
+Will return a I<HASH> reference blessed into the I<Apache::Table>
+class when called in a scalar context with no "key" argument. This
+requires I<Apache::Table>.
+
 =item $r->err_header_out( $header, [$value] )
 
 Change the value of an error response header, or create a new one.
@@ -766,12 +830,16 @@ like this:
   $r->send_fd(FILE);
   close(FILE);
 
-=item $r->internal_redirect_handler( $newplace )
+=item $r->internal_redirect( $newplace )
 
 Redirect to a location in the server namespace without 
 telling the client. For instance:
 
-   $r->internal_redirect_handler("/home/sweet/home.html");
+   $r->internal_redirect("/home/sweet/home.html");
+
+=item $r->internal_redirect_handler( $newplace )
+
+Same as I<internal_redirect>, but the I<handler> from C<$r> is preserved.
 
 =item $r->custom_response($code, $uri)
 
@@ -801,7 +869,7 @@ Example:
         #ErrorDocument 401 "sorry, go away"
         #</Location>
 
-        #$r->custom_response(AUTH_REQUIRED, qq("sorry, go away"));
+        #$r->custom_response(AUTH_REQUIRED, "sorry, go away");
 
         return AUTH_REQUIRED;
     }
@@ -853,6 +921,8 @@ kill_timeout() will disarm either variety of timeout.
 
 reset_timeout() resets the timeout in progress.
 
+=item $r->post_connection($code_ref)
+
 =item $r->register_cleanup($code_ref)
 
 Register a cleanup function which is called just before $r->pool is
@@ -863,6 +933,10 @@ destroyed.
        warn "registered cleanup called for ", $r->uri, "\n";
    });
 
+The I<post_connection> method is simply an alias for I<register_cleanup>, 
+as this method may be used to run code after the client connection is closed,
+which may not be a I<cleanup>.
+
 =back
 
 =head1 CGI SUPPORT
@@ -872,34 +946,6 @@ type of interface.
 
 =over 4
 
-=item $r->cgi_env
-
-Return a %hash that can be used to set up a standard CGI environment.
-Typical usage would be:
-
-   %ENV = $r->cgi_env
-
-B<NOTE:> The $ENV{GATEWAY_INTERFACE} is set to C<'CGI-Perl/1.1'> so
-you can say:
-
-   if($ENV{GATEWAY_INTERFACE} =~ /^CGI-Perl/) {
-       # do mod_perl stuff
-   }
-   else {
-       # do normal CGI stuff
-   }
-
-When given a key => value pair, this will set an environment variable.
-
-   $r->cgi_env(REMOTE_GROUP => "camels");
-
-=item $r->cgi_var($key);
-
-Calls $r->cgi_env($key) in a scalar context to prevent the mistake
-of calling in a list context.
-
-   my $doc_root = $r->cgi_env('DOCUMENT_ROOT');
-
 =item $r->send_cgi_header()
 
 Take action on certain headers including I<Status:>, I<Location:> and
@@ -979,7 +1025,9 @@ Apache C API notes at C<http://www.apache.org/docs/>
 
 =head1 AUTHORS
 
-Gisle Aas <aas@sn.no> and Doug MacEachern <dougm@osf.org>
+Perl interface to the Apache C API written by Doug MacEachern
+with contributions from Gisle Aas, Andreas Koenig, Eric Bartley, 
+Rob Hartill, Gerald Richter, Salvador Ortiz and others. 
 
 =cut
 
