@@ -7,6 +7,7 @@ use File::Copy ();
 
 *import = \&Exporter::import;
 @Apache::ExtUtils::EXPORT = qw(command_table);
+my $errsv = "";
 
 sub command_table {
     my($class, $cmds);
@@ -23,12 +24,18 @@ sub command_table {
 	require "$file.pm"; #so we can see prototypes
     };
     if ($@) {
+	unless ($@ =~ /Can.t locate /) {
+	    $errsv = $@;
+	}
 	require ExtUtils::testlib;
         ExtUtils::testlib->import;
 	require lib;
 	my $lib = "lib";#hmm, lib->import + -w == Unquoted string "lib" ...
 	$lib->import('./lib');
-	require $class;
+	eval { require $class };
+	if ($@ and $@ !~ /Can.t locate /) {
+	    $errsv ||= $@;
+	}
     }
     unless (-e "$file.xs.orig") {
         File::Copy::cp("$file.xs", "$file.xs.orig");
@@ -85,8 +92,8 @@ sub xs_cmd_table {
 	}
 	elsif(ref($cmd) eq "HASH") {
 	    $name = $cmd->{name};
-	    $sub = $cmd->{func};
-	    $sub = join '::', $class, $cmd->{func} unless defined &$sub;
+	    $sub = $cmd->{func} || $cmd->{name};
+	    $sub = join '::', $class, $sub unless defined &$sub;
 	    $cmd_data = $cmd->{cmd_data};
 	    $req_override = $cmd->{req_override};
 	    $desc = $cmd->{errmsg};
@@ -101,6 +108,7 @@ sub xs_cmd_table {
 	    if($name =~ s:^/::) {
 		$name .= "_END";
 	    }
+	    $sub = join '::', $class, $name;
 	}
 	$sub ||= join '::', $class, $name;
 	$req_override ||= "OR_ALL";
@@ -116,7 +124,10 @@ sub xs_cmd_table {
 	    }
 	}
 	$desc ||= "1-3 value(s) for $name";
-
+	unless ($args_how) {
+	    $errsv ||= $@;
+	    die "Can't determine prototype for `$sub': $errsv";
+	}
 	(my $cname = $name) =~ s/\W/_/g;
 	$infos .= cmd_info($cname, $sub, $cmd_data, $args_how);
 	$cmdtab .= <<EOF;
@@ -127,6 +138,18 @@ sub xs_cmd_table {
 EOF
     }
 
+    my $dir_merger = $class->can('DIR_MERGE') ?
+	"perl_perl_merge_dir_config" : "NULL";
+
+    my $dir_create = $class->can('DIR_CREATE') ?
+	"perl_perl_create_dir_config" : "NULL";
+
+    my $server_merger = $class->can('SERVER_MERGE') ?
+	"perl_perl_merge_srv_config" : "NULL";
+
+    my $server_create = $class->can('SERVER_CREATE') ?
+	"perl_perl_create_srv_config" : "NULL";
+
     return <<EOF;
 #include "modules/perl/mod_perl.h"
 
@@ -137,6 +160,7 @@ static mod_perl_perl_dir_config *newPerlConfig(pool *p)
 	    palloc(p, sizeof (mod_perl_perl_dir_config));
     cld->obj = Nullsv;
     cld->class = "$class";
+    register_cleanup(p, cld, perl_perl_cmd_cleanup, null_cleanup);
     return cld;
 }
 
@@ -169,9 +193,9 @@ module MODULE_VAR_EXPORT XS_${modname} = {
     STANDARD_MODULE_STUFF,
     NULL,               /* module initializer */
     create_dir_config_sv,  /* per-directory config creator */
-    NULL,   /* dir config merger */
+    $dir_merger,   /* dir config merger */
     create_srv_config_sv,       /* server config creator */
-    NULL,        /* server config merger */
+    $server_merger,        /* server config merger */
     mod_cmds,               /* command table */
     NULL,           /* [7] list of handlers */
     NULL,  /* [2] filename-to-URI translation */
@@ -190,6 +214,7 @@ module MODULE_VAR_EXPORT XS_${modname} = {
 MODULE = $class		PACKAGE = $class
 
 BOOT:
+    XS_${modname}.name = "$class";
     add_module(&XS_${modname});
     stash_mod_pointer("$class", &XS_${modname});
 
