@@ -3,8 +3,7 @@ use Apache ();
 #use strict; #eval'd scripts will inherit hints
 use Apache::Constants qw(:common &OPT_EXECCGI &REDIRECT);
 
-#$Id: Registry.pm,v 1.12 1998/07/08 23:00:41 dougm Exp $
-$Apache::Registry::VERSION = (qw$Revision: 1.12 $)[1];
+$Apache::Registry::VERSION = '2.01';
 
 $Apache::Registry::Debug ||= 0;
 # 1 => log recompile in errorlog
@@ -38,15 +37,14 @@ sub handler {
 	if $Debug && $Debug & 4;
 
     if (-r $filename && -s _) {
+	if (-d _) {
+	    return DECLINED; 
+	}
 	if (!($r->allow_options & OPT_EXECCGI)) {
 	    $r->log_reason("Options ExecCGI is off in this directory",
 			   $filename);
 	    return FORBIDDEN;
  	}
-	if (-d _) {
-	    $r->log_reason("attempt to invoke directory as script", $filename);
-	    return FORBIDDEN;
-	}
 	unless (-x _ or $Is_Win32) {
 	    $r->log_reason("file permissions deny server execution",
 			   $filename);
@@ -55,17 +53,18 @@ sub handler {
 
 	my $mtime = -M _;
 
+	my $uri = $r->uri;
+	$uri = "/__INDEX__" if $uri eq "/";
 	# turn into a package name
 	$r->log_error(sprintf "Apache::Registry::handler examining %s",
-		      $r->uri) if $Debug && $Debug & 4;
+		      $uri) if $Debug && $Debug & 4;
 	my $script_name = $r->path_info ?
-	    substr($r->uri, 0, length($r->uri)-length($r->path_info)) :
-		$r->uri;
+	    substr($uri, 0, length($uri)-length($r->path_info)) :
+		$uri;
 
 	if($Apache::Registry::NameWithVirtualHost) {
-	    my $srv = $r->server;
-	    $script_name = join "", $srv->server_hostname, $script_name
-		if $srv->is_virtual;
+	    my $name = $r->get_server_name;
+	    $script_name = join "", $name, $script_name if $name;
 	}
 
 	# Escape everything into valid perl identifiers
@@ -95,14 +94,8 @@ sub handler {
  	} else {
 	    $r->log_error("Apache::Registry::handler reading $filename")
 		if $Debug && $Debug & 4;
-	    my($sub);
-	    {
-		my $fh = Apache::gensym(__PACKAGE__);
-		open $fh, $filename;
-		local $/;
-		$sub = <$fh>;
-		$sub = parse_cmdline($sub);
-	    }
+	    my $sub = $r->slurp_filename;
+	    $sub = parse_cmdline($$sub);
 
 	    # compile this subroutine into the uniq package name
             $r->log_error("Apache::Registry::handler eval-ing") if $Debug && $Debug & 4;
@@ -129,7 +122,7 @@ sub handler {
 	    $r->stash_rgy_endav($script_name);
 	    if ($@) {
 		$r->log_error($@);
-		$@{$r->uri} = $@;
+		$@{$uri} = $@;
 		return SERVER_ERROR unless $Debug && $Debug & 2;
 		return Apache::Debug::dump($r, SERVER_ERROR);
 	    }
@@ -140,14 +133,14 @@ sub handler {
 
 	my $cv = \&{"$package\::handler"};
 	eval { &{$cv}($r, @_) } if $r->seqno;
-	chdir $Apache::Server::CWD;
+	$r->chdir_file("$Apache::Server::CWD/");
 	$^W = $oldwarn;
 
 	my $errsv = "";
 	if($@) {
 	    $errsv = $@;
 	    $@ = ''; #XXX fix me, if we don't do this Apache::exit() breaks
-	    $@{$r->uri} = $errsv;
+	    $@{$uri} = $errsv;
 	}
 
 	if($errsv) {
@@ -182,7 +175,7 @@ my(%switches) = (
 		    "enable with 'PerlTaintCheck On'\n")
 	   unless $Apache::__T; "";
    },
-   'w' => sub { 'BEGIN {$^W = 1;}' },
+   'w' => sub { 'BEGIN {$^W = 1;}; $^W = 1;' },
 );
 
 sub parse_cmdline {
