@@ -5,7 +5,7 @@ use strict;
 use Apache::Registry ();
 use Apache::Constants qw(OPT_EXECCGI);
 @Apache::RegistryLoader::ISA = qw(Apache::Registry);
-$Apache::RegistryLoader::VERSION = (qw$Revision: 1.7 $)[1];
+$Apache::RegistryLoader::VERSION = '1.90';
 
 sub new { 
     my $class = shift;
@@ -19,7 +19,15 @@ sub handler {
 	if(my $func = $self->{trans}) {
 	    no strict 'refs';
 	    $filename = &{$func}($uri);
-	}
+        } else {
+
+      # warn user if translate process fails,
+          warn "RegistryLoader: Cannot translate the URI $uri
+              into a real path to the filename. Please refer to the 
+              manpage for more information
+              or use the complete method's call like:
+              \$rl->handler(uri,filname);\n";
+        }
     }
 
     #warn "RegistryLoader: uri=$uri, filename=$filename\n";
@@ -36,7 +44,17 @@ sub handler {
 
 #override Apache class methods called by Apache::Registry
 #normally only available at request-time via blessed request_rec pointer
+sub slurp_filename {
+    my $r = shift;
+    my $filename = $r->filename;
+    my $fh = Apache::gensym(__PACKAGE__);
+    open $fh, $filename;
+    local $/;
+    my $code = <$fh>;
+    return \$code;
+}
 
+sub get_server_name {}
 sub filename { shift->{filename} }
 sub uri { shift->{uri} }
 sub status {200}
@@ -45,6 +63,7 @@ sub log_error { shift; die @_ if $@; warn @_; }
 *log_reason = \&log_error; 
 sub allow_options { OPT_EXECCGI } #will be checked again at run-time
 sub clear_rgy_endav {}
+sub stash_rgy_endav {}
 sub request {}
 sub seqno {0} 
 sub server { shift }
