@@ -13,12 +13,30 @@ use Config;
 #once it is sane, we'll use these methods in Makefile.PL
 
 $VERSION = '0.01';
+sub IS_MOD_PERL_BUILD () {-e "../lib/mod_perl.pm"}
+my $Is_Win32 = ($^O eq "MSWin32");
 
 sub new {
     my $class = shift;
     my $dir;
-    for (@INC) {
-	last if -d ($dir = "$_/auto/Apache/include");
+
+    if(IS_MOD_PERL_BUILD) {
+	eval {
+	    require "../lib/Apache/MyConfig.pm";
+	};
+	print $@ if $@;
+	unless ($@) {
+	    $dir = $Apache::MyConfig::Setup{Apache_Src};
+	    for ($dir, "../$dir", "../../$dir") {
+		last if -d ($dir = $_);
+	    }
+	}
+    }
+
+    unless ($dir) {
+	for (@INC) {
+	    last if -d ($dir = "$_/auto/Apache/include");
+	}
     }
 
     bless {
@@ -27,6 +45,33 @@ sub new {
     }, $class;
 }
 
+sub mmn_eq {
+    my($class, $dir) = @_;
+    return 1 if $Is_Win32; #just assume, till Apache::src works under win32 
+    my $instsrc;
+    {
+	local @INC = grep { !/blib/ } @INC;
+	my $instdir;
+        for (@INC) { 
+            last if -d ($instdir = "$_/auto/Apache/include"); 
+        } 
+	$instsrc = $class->new(dir => $instdir);
+    }
+    my $targsrc = $class->new($dir ? (dir => $dir) : ()); 
+ 
+    my $inst_mmn = $instsrc->module_magic_number; 
+    my $targ_mmn = $targsrc->module_magic_number; 
+
+    unless ($inst_mmn && $targ_mmn) {
+	return 0;
+    }
+    if ($inst_mmn == $targ_mmn) {
+	return 1;
+    }
+    print "Installed MMN $inst_mmn does not match target $targ_mmn\n";
+    return 0;
+}
+
 sub default_dir {
     eval { require Apache::MyConfig };
     return $@ ? 
@@ -87,10 +132,16 @@ sub module_magic_number {
     my $d = asrc(shift || $self->dir);
 
     #return $mcache{$d} if $mcache{$d};
-    my $fh = IO::File->new("$d/http_config.h") or return undef;
+    my $fh;
+    for (qw(ap_mmn.h http_config.h)) {
+	last if $fh = IO::File->new("$d/$_");
+    }
+    return 0 unless $fh;
+
     my $n;
+    my $mmn_pat = join "|", qw(MODULE_MAGIC_NUMBER_MAJOR MODULE_MAGIC_NUMBER);
     while(<$fh>) {
-	if(s/^#define\s+MODULE_MAGIC_NUMBER\s+(\d+).*/$1/) {
+	if(s/^#define\s+($mmn_pat)\s+(\d+).*/$2/) {
 	   chomp($n = $_);
 	   last;
        }
@@ -137,8 +188,6 @@ sub httpd_version {
     return $version;
 }
 
-my $Is_Win32 = ($^O eq "MSWin32");
-
 sub typemaps {
     my $typemaps = [];
     
@@ -149,6 +198,10 @@ sub typemaps {
 	    last;
 	}
     }
+    if(IS_MOD_PERL_BUILD) {
+	push @$typemaps, "../Apache/typemap";
+    }
+
     return $typemaps;
 }
 
@@ -157,10 +210,16 @@ sub inc {
     my $src  = $self->dir;
     my $main = $self->main;
     my $os = $Is_Win32 ? "win32" : "unix";
-    my @inc = ("-I$src", "-I$main");
+    my @inc = ("-I$src", "-I$src/modules/perl", "-I$main");
     for ("src/regex", "$src/os/$os") {
 	push @inc, "-I$_" if -d $_;
     }
+    my $ssl_dir = "$src/../ssl/include";
+    unless (-d $ssl_dir) {
+	eval { require Apache::MyConfig };
+	$ssl_dir = "$Apache::MyConfig::Setup{SSL_BASE}/include";
+    }
+    push @inc, "-I$ssl_dir" if -d $ssl_dir;
     return "@inc";
 }
 
