@@ -1,13 +1,22 @@
 package mod_perl;
 use 5.003_97;
 use strict;
+use DynaLoader ();
 
 BEGIN {
-    $mod_perl::VERSION = "1.14";
+    $mod_perl::VERSION = "1.18";
 }
 
-sub subversion {
-    print qq( -DSERVER_SUBVERSION=\\"mod_perl/$mod_perl::VERSION\\" );
+sub boot {
+    my($class, $version) = @_;
+    no strict 'refs';
+    *{$class.'::dl_load_flags'} = DynaLoader->can('dl_load_flags');
+    if ($ENV{MOD_PERL}) {
+        (defined &{$class.'::bootstrap'} ?
+         \&{$class.'::bootstrap'} :
+         \&DynaLoader::bootstrap)->
+             ($class, $version);
+    }
 }
 
 sub hook {
@@ -22,15 +31,6 @@ sub hook {
 sub import {
     my $class = shift;
 
-    #so we can say EXTRA_CFLAGS = `perl -Mmod_perl -e subversion`
-    unless(exists $ENV{MOD_PERL}) {
-	*main::subversion = \&subversion;
-	return;
-    }
-
-    $ENV{MOD_PERL} = $mod_perl::VERSION;
-    $ENV{GATEWAY_INTERFACE} = "CGI-Perl/1.1";
-
     return unless @_;
 
     if($_[0] =~ /^\d/) {
