@@ -1,3 +1,5 @@
+#PerlOpmask default
+
 <IfModule mod_dll.c>
 LoadModule perl_module modules/ApacheModulePerl.dll
 </IfModule>
@@ -7,17 +9,14 @@ AddType text/perl-module .pm
 
 Action text/perl-module /perl/action.pl
 
+PerlRequire docs/startup.pl
+PerlRequire docs/stacked.pl
+
+PerlSetEnv Tops Uno
+PerlChildInitHandler "sub { die 'PerlSetEnv busted' unless $ENV{Tops} eq 'Uno' }"
+
 PerlPassEnv TEST_PERL_DIRECTIVES
 
-#these three are passed to perl_parse(), 
-#which happens before <Perl> sections are processed
-#optionally, they can be inside <Perl>, however for testing we want
-#warnings and taint checks on while processing <Perl>
-#besides that, we rely on the PerlScript below to set @INC to our blib
-
-PerlScript docs/startup.pl
-PerlScript docs/stacked.pl
-
 #-Tw
 PerlTaintCheck On
 PerlWarn On
@@ -27,6 +26,20 @@ PerlSetEnv KeyForPerlSetEnv OK
 
 <Perl>
 #!perl
+use Apache ();
+use Apache::Registry ();
+use Apache::RegistryNG ();
+
+for (qw(TEST NOCHANCE)) {
+    if(Apache->define($_)) {
+	print "IfDefine $_\n";
+    }
+}
+
+Apache::Server->register_cleanup(sub { 
+    warn "Apache::Server registered cleanup called for $$\n";
+    0;
+});
 
 if($ENV{TEST_PERL_DIRECTIVES}) {
     #t/TestDirectives/TestDirectives.pm
@@ -67,12 +80,9 @@ EOF
 
 $My::config_is_perl = 1;
 
-#use Apache::Constants qw(MODULE_MAGIC_NUMBER);
-use IO::Handle ();
-use Cwd qw(fastcwd);
-my $dir = join "/", fastcwd, "t";
+my $dir = $Apache::Server::CWD;
+$dir .= "/t"; # if -d "t";
 my $Is_Win32 = ($^O eq "MSWin32");
-#my $mmn = MODULE_MAGIC_NUMBER;
 
 sub prompt ($;$) {
     my($mess,$def) = @_;
@@ -98,11 +108,12 @@ if($User eq "root") {
 }
 print "Will run tests as User: '$User' Group: '$Group'\n";
 
-require 't/net/config.pl';
+require 'net/config.pl';
 my $srv = $net::httpserver;
 ($srv = $net::httpserver) =~ s/\D+$//;
 $Port = (split ":", $srv, 2)[1];
 $Port ||= 8529;
+print "Will bind to Port $Port\n";
 
 $DocumentRoot = "$dir/docs";
 $ServerName = "localhost";
@@ -115,10 +126,14 @@ for (qw(/perl /cgi-bin /dirty-perl /perl_xs)) {
 
 my @mod_perl = (
     SetHandler  => "perl-script",
-    PerlHandler => "Apache::Registry",
+    PerlHandler => "Apache::RegistryNG->handler",
     Options     => "ExecCGI",
 );
 
+$Location{"/dirmagic"} = {
+    PerlHandler => "My::DirIndex",
+};
+
 $Location{"/dirty-perl"} = { 
     SetHandler => "perl-script",
     PerlHandler => "Apache::PerlRun",
@@ -146,20 +161,51 @@ $Location{"/cgi-bin"} = {
     Options    => "ExecCGI",
 };
 
-$VirtualHost{"localhost"} = {
-    Location => {
-	"/perl/io" => {
-	    @mod_perl,
-	    PerlSendHeader => "On",
-	    PerlSetupEnv   => "On",
+#just make sure we can parse this
+
+chomp(my $hostname = `hostname`);
+if($hostname =~ /ramona/) {
+    $NameVirtualHost = '209.228.11.20:8529';
+    $VirtualHost{"$NameVirtualHost:8529"} = [
+       {
+	   ServerName   => 'ramona.criticalpath.net',
+	   DocumentRoot => '/home/ix',
+	   ServerAdmin => 'ix',
+       },
+       {
+	   ServerName   => 'ramona.eng.cp.net',
+	   DocumentRoot => '/home/dougm',
+	   ServerAdmin => 'dougm',
+       },
+    ];
+
+    $Location{"/perl/io"} = {
+	@mod_perl,
+	PerlSendHeader => "On",
+	PerlSetupEnv   => "On",
+    };
+    $Location{"/perl_xs/io"} = {
+	@mod_perl,
+	PerlSendHeader => "On",
+	PerlSetupEnv   => "On",
+    };
+}
+else {
+    $VirtualHost{"localhost"} = {
+	Location => {
+	    "/perl/io" => {
+		@mod_perl,
+		PerlSendHeader => "On",
+		PerlSetupEnv   => "On",
+	    },
+	    "/perl_xs/io" => {
+		@mod_perl,
+		PerlSendHeader => "On",
+		PerlSetupEnv   => "On",
+	    },
 	},
-	"/perl_xs/io" => {
-	    @mod_perl,
-	    PerlSendHeader => "On",
-	    PerlSetupEnv   => "On",
-	},
-    },
-};
+    };
+}
 
 #$Location{"/perl/io"} = {
 #    @mod_perl,
@@ -180,8 +226,8 @@ for (qw(status info)) {
     };
 }
 
-$ErrorLog = "logs/mod_perl_error_log";
-$PidFile  = "logs/mod_perl_httpd.pid";
+$ErrorLog = "logs/error_log";
+$PidFile  = "logs/httpd.pid";
 
 $AccessConfig = $TypesConfig = $ScoreBoardFile = "$dir/docs/null.txt";
 
@@ -209,5 +255,10 @@ $Location{"/chain"} = {
     PerlHandler => [map { "Stacked::$_" } qw(one two three four)],
 };
 
+$Location{"/death"} = {
+    @mod_perl,
+    PerlHandler => "Apache::Death",
+};
+
 </Perl>
 
