@@ -1,16 +1,47 @@
+#! /usr/local/bin/perl
+
+unless (defined $ENV{MOD_PERL}) {
+    die "\$ENV{MOD_PERL} not set!";
+}
+
 BEGIN {
     #./blib/lib:./blib/arch
     use ExtUtils::testlib;
 
-    use lib './t/docs';
+    use lib map { "$Apache::Server::CWD/$_" } qw(t/docs blib/lib blib/arch);
     require "blib.pl" if -e "./t/docs/blib.pl";
-    $Apache::ServerStarting or warn "Server is not starting !?\n";
+    $Apache::Server::Starting or warn "Server is not starting !?\n";
+    \$Apache::Server::Starting == \$Apache::ServerStarting or 
+	warn "GV alias broken\n";
+    \$Apache::Server::ReStarting == \$Apache::ServerReStarting or 
+	warn "GV alias broken\n";
 }
 
+if (-e "t/docs/local.pl") {
+    eval {
+	require "local.pl"; 
+    }; $@='' if $@;
+}
+
+# BSD/OS 3.1 gets confused with some dynamically loaded code inside evals,
+# so make sure IO::File is loaded here, rather than later within an eval.
+# this should not harm any other platforms, since IO::File will be used
+# by them anyhow.
+use IO::File ();
+
+use Apache ();
+use Apache::Registry ();
+unless ($INC{'Apache.pm'} =~ /blib/) {
+    die "Wrong Apache.pm loaded: $INC{'Apache.pm'}";
+}
+
+Apache::Constants->export(qw(HTTP_MULTIPLE_CHOICES));
+
 eval {
     require Apache::PerlRunXS;
 }; $@ = '' if $@;
 
+
 {
     last;
     Apache::warn("use Apache 'warn' is ok\n");
@@ -30,7 +61,7 @@ eval {
 
 $Apache::DoInternalRedirect = 1;
 $Apache::ERRSV_CAN_BE_HTTP  = 1;
-
+$Apache::Server::AddPerlVersion = 1;
 #warn "ServerStarting=$Apache::ServerStarting\n";
 #warn "ServerReStarting=$Apache::ServerReStarting\n";
 
@@ -45,32 +76,22 @@ if($ENV{PERL_TEST_NEW_READ}) {
     *Apache::READ = \&Apache::new_read;
 }
 
-$ENV{KeyForPerlSetEnv} eq "OK" or warn "PerlSetEnv is broken\n";
-
-#test Apache::RegistryLoader
-{
-    use Apache::RegistryLoader ();
-    use DirHandle ();
-    use strict;
-    
-    my $rl = Apache::RegistryLoader->new(trans => sub {
-	my $uri = shift; 
-	$Apache::Server::CWD."/t/net${uri}";
-    });
-
-    my $d = DirHandle->new("t/net/perl");
-
-    for my $file ($d->read) {
-	next if $file eq "hooks.pl"; 
-	next unless $file =~ /\.pl$/;
-	Apache->untaint($file);
-	my $status = $rl->handler("/perl/$file");
-	unless($status == 200) {
-	    die "pre-load of `/perl/$file' failed, status=$status\n";
-	}
-    }
+unless($ENV{KeyForPerlSetEnv} and 
+       $ENV{KeyForPerlSetEnv} eq "OK") {
+    warn "PerlSetEnv is broken\n";
 }
 
+%net::callback_hooks = ();
+require "net/config.pl";
+if($net::callback_hooks{PERL_SAFE_STARTUP}) {
+    eval "open \$0";
+    unless ($@ =~ /open trapped by operation mask/) {
+	die "opmask not set";
+    }
+}
+else {
+    require "docs/rl.pl";
+}
 #for testing perl mod_include's
 
 $Access::Cnt = 0;
@@ -79,13 +100,16 @@ sub main::access { print ++$Access::Cnt }
 
 $ENV{GATEWAY_INTERFACE} =~ /^CGI-Perl/ or die "GATEWAY_INTERFACE not set!";
 
+sub Outside::imported {4}
+
 #will be redef'd during tests
 sub PerlTransHandler::handler {-1}
 
 #for testing PERL_HANDLER_METHODS
 #see httpd.conf and t/docs/LoadClass.pm
 
-use LoadClass ();
+require "docs/LoadClass.pm";
+
 sub MyClass::method ($$) {
     my($class, $r) = @_;  
     #warn "$class->method called\n";
@@ -112,9 +136,12 @@ $MyClass::Object = bless {}, "MyClass";
 
 sub My::child_init {
     my $r = shift;
-    my $s = $r->server;
-    my $sa = $s->server_admin;
-    $s->warn("[notice] child_init for process $$, report any problems to $sa\n");
+    eval {
+      my $s = $r->server;
+      my $sa = $s->server_admin;
+      $s->warn("[notice] child_init for process $$, report any problems to $sa\n");
+    }; $@='' if $@;
+    0;
 }
 
 sub My::child_exit {
@@ -140,7 +167,7 @@ sub Apache::AuthenTest::handler {
 
     my $user = lc $r->connection->user;
     $r->notes("DoAuthenTest", 1);
-
+    
     unless($user eq "dougm" and $sent_pwd eq "mod_perl") {
         $r->note_basic_auth_failure;
         return AUTH_REQUIRED;
@@ -149,6 +176,26 @@ sub Apache::AuthenTest::handler {
     return OK;                       
 }
 
+use Apache::Constants qw(DECLINED DIR_MAGIC_TYPE);
+
+sub My::DirIndex::handler {
+    my $r = shift;
+    return DECLINED unless $r->content_type and 
+	$r->content_type eq DIR_MAGIC_TYPE;
+    require DirHandle;
+    my $dh = DirHandle->new($r->filename) or die $!;
+    my @entries = $dh->read;
+    my $x = @entries;
+    $r->send_http_header('text/plain');
+    print "1..$x\n";
+    my $i = 1;
+    for my $e (@entries) {
+	print "ok $i ($e)\n";
+	++$i;
+    }
+    1;
+}
+
 sub My::ProxyTest::handler {
     my $r = shift;
     unless ($r->proxyreq and $r->uri =~ /proxytest/) {
@@ -178,6 +225,31 @@ END {
     warn "[notice] END block called for startup.pl\n";
 }
 
+package Apache::Death;
+my $say_ok = <<EOF;
+*** The following [error] is expected, no cause for alarm ***
+EOF
+
+sub handler {
+    my $r = shift;
+
+    my $args = $r->args || "";
+    if ($args =~ /die/) {
+	warn $say_ok;
+	delete $INC{"badsyntax.pl"};
+	require "badsyntax.pl";  # contains syntax error
+    }
+    if($args =~ /croak/) {
+	warn $say_ok;
+        Carp::croak("Apache::Death");
+    }
+
+    $r->content_type('text/html');
+    $r->send_http_header();
+    print "<h1>Script completed</h1>\n";
+    return 0;
+}
+
 package Destruction;
 
 sub new { bless {} }
