@@ -7,7 +7,7 @@ Apache->register_cleanup(sub {0});
 my $r;
 
 if(Apache->module("Apache::Request")) {
-    $r = Apache::Request->new;
+    $r = Apache::Request->new(shift);
 }
 else {
     $r = Apache->request;
@@ -16,12 +16,12 @@ else {
 
 my $is_xs = ($r->uri =~ /_xs/);
 
-my $tests = 50;
+my $tests = 65;
 my $is_win32 = WIN32;
 $tests += 2 unless $is_win32;
 my $test_get_set = Apache->can('set_handlers') && ($tests += 4);
 my $test_custom_response = (MODULE_MAGIC_NUMBER >= 19980324) && $tests++;
-my $test_dir_config = $INC{'Apache/TestDirectives.pm'} && ($tests += 7);
+my $test_dir_config = $INC{'Apache/TestDirectives.pm'} && ($tests += 9);
 
 my $i;
 
@@ -31,6 +31,9 @@ $r->send_http_header;
 
 $r->print("1..$tests\n");
 
+test ++$i, $ENV{MOD_PERL};
+print "ENV{MOD_PERL} = $ENV{MOD_PERL}\n";
+
 #backward compat
 %ENV = $r->cgi_env;
 my $envk = keys %ENV;
@@ -50,6 +53,13 @@ test ++$i, -d $Apache::Server::CWD;
 print "\$Apache::Server::CWD == $Apache::Server::CWD\n";
 print "\$0 == $0\n";
 
+if($Apache::Server::Starting) {
+    warn "Apache::ServerStarting var is broken\n";
+}
+if($Apache::Server::ReStarting) {
+    warn "Apache::ReServerStarting var is broken\n";
+}
+
 unless ($is_win32) {
   my $ft_s = -s $INC{'Apache.pm'};
   $r->finfo;
@@ -98,6 +108,20 @@ test ++$i, keys %err_headers_out;
 $r->err_header_out('X-Die' => "uhoh"); 
 test ++$i, $r->err_header_out("X-Die") eq "uhoh";
 
+for (1..3)  {
+    test ++$i, not $r->pnotes("NO_CHANCE");
+    $r->pnotes(KEY => [qw(one two)]);
+    my $val = $r->pnotes('KEY');
+    test ++$i, $val && (ref($val) eq 'ARRAY');
+    $val = $r->pnotes;
+    test ++$i, $val && (ref($val) eq 'HASH');
+    while(my($kk,$vv) = each %$val) {
+	test ++$i, $kk && $vv;
+    }
+#    use Data::Dumper;
+#    print Dumper $val;
+}
+
 $r->notes("FOO", 1); 
 $r->notes("ANoteKey", "TRUE");
 test ++$i, $r->notes("ANoteKey");
@@ -141,6 +165,7 @@ test ++$i, $s;
 test ++$i, $s->server_admin;
 test ++$i, $s->server_hostname;
 test ++$i, $s->port;
+test ++$i, $s->timeout;
 
 for (my $srv = $r->server; $srv; $srv = $srv->next) {
     my $name = $srv->server_hostname;
@@ -150,6 +175,12 @@ for (my $srv = $r->server; $srv; $srv = $srv->next) {
 my $str = "ok $i\n";
 $r->print(\$str);
 
+test ++$i, $r->define("FOO") || 1; #just make sure we can call it
+for (qw(TEST NOCHANCE)) {
+    if(Apache->define($_)) {
+	print "IfDefine $_\n";
+    }
+}
 test ++$i, $r->module("Apache");
 test ++$i, not Apache->module("Not::A::Chance");
 test ++$i, Apache->module("Apache::Constants");
@@ -178,10 +209,19 @@ if($test_get_set) {
 }
 
 if($test_dir_config) {
+    require Data::Dumper;
     require Apache::ModuleConfig;
     my $dc = Apache::ModuleConfig->get($r);
     test ++$i, not $dc;
 
+    {
+	package Apache::TestDirectives;
+	use Apache::test 'test';
+	my $scfg = Apache::ModuleConfig->get($r->server);
+	test ++$i, $scfg;
+	test ++$i,  __PACKAGE__->isa($scfg->{ServerClass});
+	print Data::Dumper::Dumper($scfg);
+    }
     for my $cv (
 		sub {
 		    package Apache::TestDirectives;
@@ -192,7 +232,6 @@ if($test_dir_config) {
 		})
     {
         my $cfg = $cv->();
-        require Data::Dumper;
         $r->print(Data::Dumper::Dumper($cfg));
         test ++$i, "$cfg" =~ /HASH/;
         test ++$i, keys(%$cfg) >= 3;
